/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.document.library.internal.exportimport.data.handler;

import com.liferay.document.library.kernel.exception.NoSuchFolderException;
import com.liferay.document.library.kernel.model.DLFolder;
import com.liferay.document.library.kernel.service.DLFolderLocalServiceUtil;
import com.liferay.portal.kernel.dao.orm.QueryUtil;
import com.liferay.portal.kernel.repository.model.Folder;
import com.liferay.portal.repository.liferayrepository.model.LiferayFolder;
import com.liferay.portlet.documentlibrary.util.RepositoryModelUtil;

import java.util.List;

/**
 * @author Alexander Chow
 */
public class FolderUtil {

	public static Folder fetchByR_P_N(
		long groupId, long parentFolderId, String name) {

		DLFolder dlFolder = DLFolderLocalServiceUtil.fetchFolder(
			groupId, parentFolderId, name);

		if (dlFolder == null) {
			return null;
		}

		return new LiferayFolder(dlFolder);
	}

	public static Folder fetchByUUID_R(String uuid, long repositoryId) {
		DLFolder dlFolder = DLFolderLocalServiceUtil.fetchFolder(
			uuid, repositoryId);

		if (dlFolder == null) {
			return null;
		}

		return new LiferayFolder(dlFolder);
	}

	public static Folder findByPrimaryKey(long folderId)
		throws NoSuchFolderException {

		DLFolder dlFolder = DLFolderLocalServiceUtil.fetchFolder(folderId);

		return new LiferayFolder(dlFolder);
	}

	/**
	 * @deprecated As of 1.1.0, with no direct replacement
	 */
	@Deprecated
	public static List<Folder> findByR_P(
		long repositoryId, long parentFolderId) {

		List<DLFolder> dlFolders = DLFolderLocalServiceUtil.getFolders(
			repositoryId, parentFolderId, true);

		return RepositoryModelUtil.toFolders(dlFolders);
	}

	/**
	 * @deprecated As of 1.1.0, with no direct replacement
	 */
	@Deprecated
	public static List<Folder> findByRepositoryId(long repositoryId) {
		List<DLFolder> dlFolders =
			DLFolderLocalServiceUtil.getRepositoryFolders(
				repositoryId, QueryUtil.ALL_POS, QueryUtil.ALL_POS);

		return RepositoryModelUtil.toFolders(dlFolders);
	}

}