/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.document.library.internal.xstream.configurator;

import com.liferay.document.library.internal.lar.xstream.FileEntryConverter;
import com.liferay.document.library.internal.lar.xstream.FileVersionConverter;
import com.liferay.document.library.internal.lar.xstream.FolderConverter;
import com.liferay.exportimport.kernel.xstream.XStreamAlias;
import com.liferay.exportimport.kernel.xstream.XStreamConverter;
import com.liferay.exportimport.kernel.xstream.XStreamType;
import com.liferay.portal.kernel.util.ListUtil;
import com.liferay.portal.model.impl.RepositoryEntryImpl;
import com.liferay.portal.model.impl.RepositoryImpl;
import com.liferay.portlet.documentlibrary.model.impl.DLFileEntryImpl;
import com.liferay.portlet.documentlibrary.model.impl.DLFileEntryTypeImpl;
import com.liferay.portlet.documentlibrary.model.impl.DLFileShortcutImpl;
import com.liferay.portlet.documentlibrary.model.impl.DLFolderImpl;
import com.liferay.xstream.configurator.XStreamConfigurator;

import java.util.List;

import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;

/**
 * @author Mate Thurzo
 */
@Component(immediate = true, service = XStreamConfigurator.class)
public class DocumentLibraryXStreamConfigurator implements XStreamConfigurator {

	@Override
	public List<XStreamType> getAllowedXStreamTypes() {
		return null;
	}

	@Override
	public List<XStreamAlias> getXStreamAliases() {
		return ListUtil.toList(_xStreamAliases);
	}

	@Override
	public List<XStreamConverter> getXStreamConverters() {
		return ListUtil.toList(_xStreamConverters);
	}

	@Activate
	protected void activate() {
		_xStreamAliases = new XStreamAlias[] {
			new XStreamAlias(DLFileEntryImpl.class, "DLFileEntry"),
			new XStreamAlias(DLFileEntryTypeImpl.class, "DLFileEntryType"),
			new XStreamAlias(DLFileShortcutImpl.class, "DLFileShortcut"),
			new XStreamAlias(DLFolderImpl.class, "DLFolder"),
			new XStreamAlias(RepositoryImpl.class, "Repository"),
			new XStreamAlias(RepositoryEntryImpl.class, "RepositoryEntry")
		};

		_xStreamConverters = new XStreamConverter[] {
			new FileEntryConverter(), new FileVersionConverter(),
			new FolderConverter()
		};
	}

	private XStreamAlias[] _xStreamAliases;
	private XStreamConverter[] _xStreamConverters;

}